<?php
/**
 * IDEALIAGroup srl
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@idealiagroup.com so we can send you a copy immediately.
 *
 * @category   MSP
 * @package    MSP_CashOnDelivery
 * @copyright  Copyright (c) 2016 IDEALIAGroup srl (http://www.idealiagroup.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

namespace MSP\CashOnDelivery\Model;

use Magento\Payment\Model\Method\AbstractMethod;
use MSP\CashOnDelivery\Api\CashondeliveryTableInterface;

class Payment extends AbstractMethod
{
    const CODE = 'msp_cashondelivery';
    const XML_PATH_EXCLUDE_REGIONS = 'payment/msp_cashondelivery/exclude_regions';
    const XML_PATH_MIN_AMOUNT = 'payment/msp_cashondelivery/min_order_total';
    const XML_PATH_MAX_AMOUNT = 'payment/msp_cashondelivery/max_order_total';

    protected $_code = self::CODE;

    protected $_formBlockType = 'Magento\OfflinePayments\Block\Form\Checkmo';
    protected $_infoBlockType = 'MSP\CashOnDelivery\Block\Info\CashOnDelivery';

    protected $_isOffline = true;
    
    protected $cashondeliveryTableInterface;
    protected $priceCurrency;
    
    public function __construct(
            \Magento\Framework\Model\Context $context, 
            \Magento\Framework\Registry $registry, 
            \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory, 
            \Magento\Framework\Api\AttributeValueFactory $customAttributeFactory, 
            \Magento\Payment\Helper\Data $paymentData, 
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig, 
            \Magento\Payment\Model\Method\Logger $logger, 
            CashondeliveryTableInterface $cashondeliveryTableInterface, 
            \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency, 
            \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null, 
            \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null, 
            array $data = array()) {
        
        
        parent::__construct(
                $context, 
                $registry, 
                $extensionFactory, 
                $customAttributeFactory, 
                $paymentData, $scopeConfig, 
                $logger, 
                $resource, 
                $resourceCollection, 
                $data);
        $this->cashondeliveryTableInterface = $cashondeliveryTableInterface;
        $this->priceCurrency = $priceCurrency;
    }

    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        if (!parent::isAvailable($quote)) {
            return false;
        }

        $excludeRegions = $this->_scopeConfig->getValue(static::XML_PATH_EXCLUDE_REGIONS);

        if(!empty($excludeRegions)) {
            $excludeRegions = explode(',', $excludeRegions);
            foreach ($quote->getAllShippingAddresses() as $shippingAddress) {
                if (in_array($shippingAddress->getRegionId(), $excludeRegions)) {
                    return false;
                }
            }
        }
        
        if (!$this->isAmountOk($quote)) {
            return false;
        }

        return true;
    }
    
    protected function isAmountOk($quote) {
        $is_msp_cod_amount_ok_calculated = $this->_registry->registry('is_msp_cod_amount_ok_calculated');
        if (!is_null($is_msp_cod_amount_ok_calculated)) {
            return $is_msp_cod_amount_ok_calculated;
        }
        $min_amount = $this->_scopeConfig->getValue(static::XML_PATH_MIN_AMOUNT);
        $max_amount = $this->_scopeConfig->getValue(static::XML_PATH_MAX_AMOUNT);
        if (is_null($quote)) {
            return false;
        }
        if (is_null($quote->getShippingAddress())) {
            return false;
        }
        $country = $quote->getShippingAddress()->getCountryModel()->getData('iso2_code');
        $region = $quote->getShippingAddress()->getRegionModel()->getData('code');
        $cod_amount = $this->cashondeliveryTableInterface->getFee($quote->getGrandTotal(), $country, $region);
        $quote_amount = $quote->getBaseGrandTotal();

        if ($max_amount && $this->priceCurrency->round($quote_amount + $cod_amount) > $max_amount) {
            $this->_registry->register('is_msp_cod_amount_ok_calculated', false);
            return false;
        }
        if ($min_amount && $this->priceCurrency->round($quote_amount + $cod_amount) < $min_amount) {
            $this->_registry->register('is_msp_cod_amount_ok_calculated', false);
            return false;
        }
        $this->_registry->register('is_msp_cod_amount_ok_calculated', true);
        return true;
    }
}
